import {Viewer} from '@xeokit/xeokit-sdk';

interface AABB {
    0: number;
    1: number;
    2: number;
    3: number;
    4: number;
    5: number;
}

interface Entity {
    aabb: AABB;
    // 其他 entity 属性...
}

interface ObjectsKdTreeNode {
    aabb: AABB;
    left?: ObjectsKdTreeNode;
    right?: ObjectsKdTreeNode;
    entities?: Entity[];
}

interface ObjectsKdTreeConfig {
    viewer: Viewer;
    maxTreeDepth?: number;
}

export class ObjectsKdTree3 {
    private viewer: Viewer;
    private _maxTreeDepth: number;
    private _root: ObjectsKdTreeNode | null;
    private _needsRebuild: boolean;
    private _onModelLoaded: any;
    private _onModelUnloaded: any;
    private MAX_KD_TREE_DEPTH = 15

    constructor(cfg: ObjectsKdTreeConfig) {
        if (!cfg) {
            throw new Error('Parameter expected: cfg');
        }

        if (!cfg.viewer) {
            throw new Error('Parameter expected: cfg.viewer');
        }

        this.viewer = cfg.viewer;
        this._maxTreeDepth = cfg.maxTreeDepth || this.MAX_KD_TREE_DEPTH;
        this._root = null;
        this._needsRebuild = true;

        this._onModelLoaded = this.viewer.scene.on('modelLoaded', (modelId: any) => {
            this._needsRebuild = true;
        });

        this._onModelUnloaded = this.viewer.scene.on('modelUnloaded', (modelId: any) => {
            this._needsRebuild = true;
        });
    }

    get root(): ObjectsKdTreeNode | null {
        if (this._needsRebuild) {
            this._rebuild();
        }
        return this._root;
    }

    private _rebuild(): void {
        const viewer = this.viewer;
        const scene = viewer.scene;
        const depth = 0;
        this._root = {
            //@ts-ignore
            aabb: scene.getAABB(),
        };
        for (const objectId in scene.objects) {
            const entity = scene.objects[objectId];
            //@ts-ignore
            this._insertEntity(this._root, entity, depth + 1);
        }
        this._needsRebuild = false;
    }

    private _insertEntity(node: ObjectsKdTreeNode, entity: Entity, depth: number): void {
        // 实现 _insertEntity 方法的代码...
    }

    destroy(): void {
        const scene = this.viewer.scene;
        scene.off(this._onModelLoaded);
        scene.off(this._onModelUnloaded);
        this._root = null;
        this._needsRebuild = true;
    }
}
