/**
 * 模型加载起
 * 用于管理图纸/3D模型的的加载卸载初始化等操作
 */
// @ts-ignore
import {MarqueePicker, MarqueePickerMouseControl, ObjectsKdTree3, Viewer, XKTLoaderPlugin} from '@xeokit/xeokit-sdk';
import {BimShowTime} from '../index';


export class BstLoader {
    viewer: Viewer;
    loader: XKTLoaderPlugin;

    constructor(private bst: BimShowTime) {
        this.viewer = bst.viewer;
        this.loader = new XKTLoaderPlugin(this.viewer);
    }

    /**
     * 加载model,如果不设定ID,则自动生成ID
     * @param path
     * @param id
     */
    loadModel(path: string, id: string | null = null): { id: string, path: string } {
        const model = this.loader.load({
            id: id ?? '',
            src: path,
            edges: true
        });
        model.on('loaded', _ => {
            console.log(model)
            if (this.bst.is2D) {
                let camera = this.viewer.camera;
                let abc = model.aabb;
                // let xBox = Math.sqrt((abc[3] - abc[0]) * (abc[3] - abc[0]) + (abc[4] - abc[3]) * (abc[4] - abc[3]) + (abc[5] - abc[2]) * (abc[5] - abc[2]));
                // console.log('xbox', xBox);
                let xBox = 860315;
                camera.projection = 'perspective'
                // camera.perspective.near = xBox * 0.001;
                camera.perspective.far = xBox * 1;
                this.viewer.cameraControl.mouseWheelDollyRate = 15000;
                camera.up = [0, 1, 0];
                this.viewer.cameraControl.dragRotationRate = 0;
                this.viewer.cameraControl.rotationInertia = 0;
                this.viewer.cameraControl.keyboardRotationRate = 0;
            } else {
                const objectsKdTree3 = new ObjectsKdTree3({viewer: this.viewer});
                const marqueePicker = new MarqueePicker({viewer: this.viewer, objectsKdTree3});
                const marqueePickerMouseControl = new MarqueePickerMouseControl({marqueePicker});
                marqueePicker.on('clear', () => {
                    this.viewer.scene.setObjectsSelected(this.viewer.scene.selectedObjectIds, false);
                });
                marqueePicker.on('picked', (objectIds: Array<any>) => {
                    this.viewer.scene.setObjectsSelected(objectIds, true);
                    this.bst.event.marqueePicked.next(objectIds);
                    if (this.bst.state.marqueeFit) {
                        this.bst.camera.viewFit(objectIds);
                    }
                });
                marqueePickerMouseControl.setActive(true);
            }
            this.viewer.cameraFlight.flyTo(model)
        })
        let m = {id: model.id, path};
        this.bst.loadedModels.push(m);
        return m;
    }

    /**
     * 卸载一个图纸
     * @param id 图纸加载时的ID
     */
    unload(id: string): Promise<boolean> {
        return new Promise((resolve, reject) => {
            const model = this.viewer.scene.models[id];
            // @ts-ignore
            model.on('destroyed', () => {
                resolve(true)
            });
            if (model) {
                model.destroy();
            } else {
                reject(false)
            }
        });
    }

    /**
     * 获取所有已经加载的Models
     */
    getAllModels(): Array<{ id: string, path: string }> {
        return this.bst.loadedModels;
    }


}
