import {BimShowTime} from './index';
import {Annotation, AnnotationsPlugin} from '@xeokit/xeokit-sdk';

export class BstMarker {
    private marker: AnnotationsPlugin;
    private markHtml = '<div class=\'annotation-marker\' style=\'background-color: {{markerBGColor}};\'>{{glyph}}</div>'
    private contentHtml = '<div class=\'annotation-label\' style=\'background-color: {{labelBGColor}};\'><div class=\'annotation-title\'>{{title}}</div><div class=\'annotation-desc\'>{{description}}</div></div>'
    private defaultValues = {
        markerBGColor: 'red',
        glyph: 'X',
        title: 'Untitled',
        description: 'No description'
    };

    constructor(private bst: BimShowTime) {
        this.marker = new AnnotationsPlugin(bst.viewer, {
            markerHTML: this.markHtml,
            labelHTML: this.contentHtml,
            values: this.defaultValues,
            surfaceOffset: 0.1
        });
    }

    /**
     * 设置标志样式HTML
     * 可以使用变量 用 {{变量}} 的形式放在html中,后续会用value替换
     * @param content
     */
    setMarkerHtml(content: string) {
        this.markHtml = content;
    }

    /**
     * 设置弹出的html样式
     * 可以使用变量 用 {{变量}} 的形式放在html中,后续会用value替换
     * @param content
     */
    setContentHtml(content: string) {
        this.contentHtml = content;
    }

    /**
     * 设置默认的数值
     * @param values
     */
    setDefaultValues(values: any) {
        this.defaultValues = values;
    }

    /**
     * 创建一个marker
     * @param worldPos 世界坐标
     * @param value 对应的替换变量
     */
    create(worldPos: number[], value: any = null): Annotation {
        if (value == null) {
            value = this.defaultValues;
        }
        return this.marker.createAnnotation({
            id: this.bst.generateRandomString(10),
            worldPos,
            occludable: true,
            markerShown: true,
            labelShown: true,
            values: value
        });
    }

    /**
     * 获取所有的marker
     */
    getAll(): Array<Annotation> {
        return Object.values(this.marker.annotations);
    }

    /**
     * 重现所有的marker
     * @param markers
     */
    setAll(markers: Array<Annotation>) {
        for (let a of markers) {
            this.marker.createAnnotation({
                id: a.id,
                worldPos: a.worldPos,
                occludable: a.occludable,
                markerShown: a.getMarkerShown(),
                labelShown: a.getLabelShown(),
                values: a.getValues()
            });
        }
    }

    /**
     * 删除一个marker
     * @param id marker的ID
     */
    delete(id: string) {
        this.marker.destroyAnnotation(id);
    }

}
