import {BimShowTime} from './index';
import {
    AngleMeasurementsMouseControl,
    AngleMeasurementsPlugin,
    DistanceMeasurementsMouseControl,
    DistanceMeasurementsPlugin,
    PointerLens
} from '@xeokit/xeokit-sdk';

export class BstMeasurement {
    distanceMeasurements: DistanceMeasurementsPlugin;
    distanceMeasurementsMouseControl: DistanceMeasurementsMouseControl;
    angleMeasurements: AngleMeasurementsPlugin;
    angleMeasurementsMouseControl: AngleMeasurementsMouseControl;

    constructor(private bst: BimShowTime) {
        this.distanceMeasurements = new DistanceMeasurementsPlugin(bst.viewer);
        this.distanceMeasurementsMouseControl = new DistanceMeasurementsMouseControl(this.distanceMeasurements, {
            pointerLens: new PointerLens(bst.viewer)
        })
        this.angleMeasurements = new AngleMeasurementsPlugin(bst.viewer, {zIndex: 100000});
        this.angleMeasurementsMouseControl = new AngleMeasurementsMouseControl(this.angleMeasurements, {
            pointerLens: new PointerLens(bst.viewer)
        })
    }

    /**
     * 开启角度测量模式
     * !!会自动退出距离测量模式
     */
    angleMode() {
        // this.distanceMeasurementsMouseControl.deactivate();
        this.angleMeasurementsMouseControl.snapping = false;
        this.angleMeasurementsMouseControl.activate();
        this.bst.state.mode = 'angleMeasurements';
        console.log('angleModeEnable')
    }

    /**
     * 开启距离测量模式
     * !! 会自动退出角度测量模式
     */
    distanceMode() {
        this.angleMeasurementsMouseControl.deactivate();
        this.distanceMeasurementsMouseControl.snapping = false;
        this.distanceMeasurementsMouseControl.activate();
        this.bst.state.mode = 'distanceMeasurements';
        console.log('distanceModeEnable')
    }

    /**
     * 退出测量模式
     */
    exit() {
        this.angleMeasurementsMouseControl.deactivate();
        this.distanceMeasurementsMouseControl.deactivate();
        this.bst.state.mode = null;
    }
}
