import {BimShowTime} from './index';

/**
 * 构件对象操作
 */
export class BstObject {
    constructor(private bst: BimShowTime) {
    }

    /**
     * 显示构件
     * @param objectIds 构件Ids
     */
    showObject(objectIds: Array<string>): void {
        this.bst.viewer.scene.setObjectsVisible(objectIds, true);
    }

    /**
     * 隐藏构件
     * @param objectIds 构件Ids
     */
    hideObject(objectIds: Array<string>): void {
        this.bst.viewer.scene.setObjectsVisible(objectIds, false);
    }


    /**
     * 构件透明
     * @param objectIds 构件Ids
     */
    transparentObject(objectIds: Array<string>): void {
        this.bst.viewer.scene.setObjectsOpacity(objectIds, 0.2);
    }


    /**
     * 取消构件透明
     * @param objectIds 构件Ids
     */
    untransparentObject(objectIds: Array<string>): void {
        this.bst.viewer.scene.setObjectsOpacity(objectIds, 1);
    }


    /**
     * 构件高亮显示
     * @param objectIds 构件Ids
     */
    highlightObject(objectIds: Array<string>): void {
        this.bst.viewer.scene.setObjectsHighlighted(objectIds, true);
    }

    /**
     * 取消构件高亮显示
     * @param objectIds 构件Ids
     */
    unhighlightObject(objectIds: Array<string>): void {
        this.bst.viewer.scene.setObjectsHighlighted(objectIds, false);
    }


    /**
     * 只显示给定的构件
     * @param objectIds 构件Ids
     */
    onlyShowObjects(objectIds: Array<string>): void {
        this.hideObject(this.bst.viewer.scene.objectIds);
        this.showObject(objectIds);
    }

    /**
     * 选择一个构件
     * 注意: 多次调用会选择多个,同时,如果用户点击会对选择有影响
     * @param objectIds 要选择的构件Ids
     */
    selectObject(objectIds: Array<string>): void {
        this.bst.viewer.scene.setObjectsSelected(objectIds, true);
    }

    /**
     * 取消选择一个构件
     * @param objectIds 要取消选择的构件Ids
     */
    unselectObject(objectIds: Array<string>): void {
        this.bst.viewer.scene.setObjectsSelected(objectIds, false);
    }

    /**
     * 获取对应构件的相关信息
     * @param objectIds 要取消选择的构件Ids
     */
    getObjectInfo(objectIds: Array<string>): Promise<object> {
        return new Promise<object>(() => {
        })
    }

    /**
     * 根据给定的构件,选择同类构件
     * @param objectId 要取消选择的构件Ids
     * @return 同类型构件的列表 注意,包含当前作为参数的构件
     */
    getSameTypeObjects(objectId: string): Promise<Array<object>> {
        return new Promise<Array<object>>(() => {
        })
    }
}
