import {BimShowTime} from './index';
import {math, SectionPlane, SectionPlanesPlugin} from '@xeokit/xeokit-sdk';

export class BstSlice {
    private sectionPlane: SectionPlanesPlugin;

    constructor(private bst: BimShowTime) {
        this.sectionPlane = new SectionPlanesPlugin(bst.viewer, {
            overviewCanvasId: bst.sliceCanvas.id,
            overviewVisible: true
        });
        this.bst.event.mouseClick.subscribe(x => {
            if (bst.state.mode === 'slice') {
                if (x && x.worldNormal) { // Disallow SectionPlanes on point clouds, because points don't have normals
                    const sectionPlane = this.sectionPlane.createSectionPlane({
                        pos: x.worldPos,
                        dir: math.mulVec3Scalar(x.worldNormal, -1)
                    });
                    this.sectionPlane.showControl(sectionPlane.id);
                }
            }
        })
    }

    /**
     * 进入剖切模式,点击创建剖切面
     */
    enter() {
        this.bst.state.mode = 'slice';
    }

    /**
     * 退出剖切模式,会保留创建的剖切面
     */
    exit() {
        this.bst.state.mode = null;
    }

    /**
     * 获取所有剖切面信息
     */
    getAll(): { [key: string]: SectionPlane } {
        return this.sectionPlane.sectionPlanes;
    }

    /**
     * 删除某个剖切面
     * @param id 剖切面的ID
     */
    delete(id: string) {
        this.sectionPlane.destroySectionPlane(id);
    }

    /**
     * 清除所有的剖切面
     */
    clear() {
        this.sectionPlane.clear();
    }
}
